package com.abcseo.ws2300;

import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.mortbay.http.HttpContext;
import org.mortbay.http.HttpServer;
import org.mortbay.http.SocketListener;
import org.mortbay.http.handler.ResourceHandler;
import org.mortbay.jetty.servlet.ServletHandler;

import sun.misc.Signal;
import sun.misc.SignalHandler;

/*
 * Program to read and publish data from an LaCrosse 23** series weather station
 * 
 * The program uses the Apache Velocity template engine to create user
 * configurable HTML page(s), these can be published either by an internal
 * webserver (Jetty) or through FTP to a remote web server.
 * 
 * Note: uses Java 5.0
 * 
 * @author david george
 * 
 * (c) 10-May-2005 David George
 * This program is published under the GNU General Public license
 */
public class OpenJ2300 {
	static Configuration config;
	private static Logger logger = Logger.getLogger("com.abcseo");

	public static void main(String[] args) {
		OpenJ2300 oj2300 = new OpenJ2300();
		config = Configuration.getInstance();

		// Set logging level
		Level level = Level.parse(config.getLogLevel());
		logger.setLevel(level);
		
		WS2300 weatherStation = new WS2300();
		

		// Start the Webcam
		String device = config.getWebCamDevice();
		String docRoot = config.getDocRoot();
		if (device.equals("")) {
			logger.log(Level.INFO,
					"No webcam device configured in properties file");
		} else {
			try {
				WebCam wc = new WebCam(device, docRoot);
				wc.start();
				config.addThread(wc);
			} catch (Exception e) {
				logger.log(Level.SEVERE, "Error connecting to Webcam " + e);
			}
		}

		// Start publishing weather data
		PublishThread pt = new PublishThread(weatherStation, docRoot);
		pt.start();
		config.addThread(pt);

		// Start the internal webserver to serve weather data

		short port = config.getWebServerPort();

		if (port > 0) {
			try {
				// Create the server and add a listener on a port
				HttpServer server = new HttpServer();
				SocketListener listener = new SocketListener();
				// listener.setHost ( "myhost" );
				listener.setPort(config.getWebServerPort());
				listener.setMinThreads(5);
				listener.setMaxThreads(250);
				server.addListener(listener);

				// Create a context
				HttpContext context = new HttpContext();
				context.setContextPath("/");
				server.addContext(context);

				// Create a servlet container
				ServletHandler handler = new ServletHandler();
				context.addHandler(handler);
				context.setResourceBase(docRoot);

				// Map a servlet onto the container
				handler.addServlet("Configuration", "/Shutdown",
						"com.abcseo.ws2300.servlet.ShutdownServlet");

				context.addHandler(new ResourceHandler());

				// Start the http server
				server.start();
			} catch (Exception e) {
				logger.log(Level.SEVERE,
						"Error starting the webserver on port " + port
								+ " Exception " + e);
			}
		} else {
			// no webserver to initial shutdown listen on SIGINT CTL-C
			new SignalHelper("INT", config);
		}
	}
}

/**
 * <p>
 * Private helper class for <code>SignalInterceptor</code>.
 * </p>
 * 
 * <p>
 * This class exists separately from <code>SignalInterceptor</code> to permit
 * graceful handling of LinkageErrors when the <code>sun.misc.Signal*</code>
 * classes don't exist.
 * </p>
 */
final class SignalHelper implements SignalHandler {
    private final SignalHandler oldHandler;
	private static Logger logger = Logger.getLogger("com.abcseo");
    Configuration config;

    SignalHelper(String signame, Configuration config) {
    	this.config = config;
        Signal signal = new Signal(signame);
        oldHandler = Signal.handle(signal, this);

;
    }

    public void handle(Signal sig) {
    	logger.log(Level.INFO, "Shutting down OpenJ2300...");
		Vector<WSThread> vt = config.getThreadList();
		
		for (int i = 0; i < vt.size(); i++) {
			WSThread t = vt.get(i);
			t.shutdown();
		}
		logger.log(Level.INFO, "OpenJ2300 shutdown complete");
		System.out.flush();
        if (oldHandler != null) {
            oldHandler.handle(sig);
        }
    }
}
